@@ -41,6 +41,7 @@ int h_close_db(struct _h_connection * conn) {
   if (conn != NULL && conn->connection != NULL) {
     if (0) {
       /* Not happening */
+      return H_ERROR_PARAMS;
 #ifdef _HOEL_SQLITE
     } else if (conn->type == HOEL_DB_TYPE_SQLITE) {
       h_close_sqlite(conn);
@@ -73,6 +74,7 @@ char * h_escape_string(const struct _h_connection * conn, const char * unsafe) {
   if (conn != NULL && conn->connection != NULL && unsafe != NULL) {
     if (0) {
       /* Not happening */
+      return NULL;
 #ifdef _HOEL_SQLITE
     } else if (conn->type == HOEL_DB_TYPE_SQLITE) {
       return h_escape_string_sqlite(conn, unsafe);
@@ -102,6 +104,7 @@ char * h_escape_string_with_quotes(const struct _h_connection * conn, const char
   if (conn != NULL && conn->connection != NULL && unsafe != NULL) {
     if (0) {
       /* Not happening */
+      return NULL;
 #ifdef _HOEL_SQLITE
     } else if (conn->type == HOEL_DB_TYPE_SQLITE) {
       return h_escape_string_with_quotes_sqlite(conn, unsafe);
@@ -137,6 +140,7 @@ int h_execute_query(const struct _h_connection * conn, const char * query, struc
   if (conn != NULL && conn->connection != NULL && query != NULL) {
     if (0) {
       /* Not happening */
+      return H_ERROR_PARAMS;
 #ifdef _HOEL_SQLITE
     } else if (conn->type == HOEL_DB_TYPE_SQLITE) {
       if (options & H_OPTION_EXEC) {
@@ -172,6 +176,7 @@ int h_execute_query_json(const struct _h_connection * conn, const char * query,
   if (conn != NULL && conn->connection != NULL && query != NULL && j_result != NULL) {
     if (0) {
       /* Not happening */
+      return H_ERROR_PARAMS;
 #ifdef _HOEL_SQLITE
     } else if (conn->type == HOEL_DB_TYPE_SQLITE) {
       return h_execute_query_json_sqlite(conn, query, j_result);
@@ -197,7 +202,7 @@ int h_execute_query_json(const struct _h_connection * conn, const char * query,
  * return H_OK on success
  */
 int h_row_add_data(struct _h_data ** row, struct _h_data * data, int cols) {
-  struct _h_data * tmp = o_realloc(*row, (cols+1)*sizeof(struct _h_data));
+  struct _h_data * tmp = o_realloc(*row, ((size_t)cols+1)*sizeof(struct _h_data));
   * row = tmp;
   if (tmp == NULL) {
     y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for h_row_add_data");
@@ -236,7 +241,7 @@ int h_row_add_data(struct _h_data ** row, struct _h_data * data, int cols) {
           ((struct _h_type_text *)tmp[cols].t_data)->value = o_malloc(((struct _h_type_text *)data->t_data)->length+sizeof(char));
           if (((struct _h_type_text *)tmp[cols].t_data)->value == NULL) {
             y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for ((struct _h_type_text *)tmp[cols].t_data)->value");
-            o_free(tmp[cols].t_data);
+            h_free(tmp[cols].t_data);
             return H_ERROR_MEMORY;
           }
           memcpy(((struct _h_type_text *)tmp[cols].t_data)->value, ((struct _h_type_text *)data->t_data)->value, (((struct _h_type_text *)data->t_data)->length+1));
@@ -255,7 +260,7 @@ int h_row_add_data(struct _h_data ** row, struct _h_data * data, int cols) {
           ((struct _h_type_blob *)tmp[cols].t_data)->value = o_malloc(((struct _h_type_blob *)data->t_data)->length);
           if (((struct _h_type_blob *)tmp[cols].t_data)->value == NULL) {
             y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for ((struct _h_type_blob *)tmp[cols].t_data)->value");
-            o_free(tmp[cols].t_data);
+            h_free(tmp[cols].t_data);
             return H_ERROR_MEMORY;
           }
           memcpy(((struct _h_type_blob *)tmp[cols].t_data)->value, ((struct _h_type_blob *)data->t_data)->value, ((struct _h_type_blob *)data->t_data)->length);
@@ -290,7 +295,7 @@ int h_row_add_data(struct _h_data ** row, struct _h_data * data, int cols) {
  * return H_OK on success
  */
 int h_result_add_row(struct _h_result * result, struct _h_data * row, int rows) {
-  result->data = o_realloc(result->data, (rows+1)*sizeof(struct _h_data *));
+  result->data = o_realloc(result->data, ((size_t)rows+1)*sizeof(struct _h_data *));
   if (result->data == NULL) {
     y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for result->data");
     return H_ERROR_MEMORY;
@@ -326,7 +331,7 @@ struct _h_data * h_query_last_insert_id(const struct _h_connection * conn) {
       /* Not happening */
 #ifdef _HOEL_SQLITE
     } else if (conn->type == HOEL_DB_TYPE_SQLITE) {
-      int last_id = h_last_insert_id_sqlite(conn);
+      long long int last_id = h_last_insert_id_sqlite(conn);
       if (last_id > 0) {
         data = h_new_data_int(last_id);
       } else {
@@ -335,7 +340,7 @@ struct _h_data * h_query_last_insert_id(const struct _h_connection * conn) {
 #endif
 #ifdef _HOEL_MARIADB
     } else if (conn->type == HOEL_DB_TYPE_MARIADB) {
-      int last_id = h_last_insert_id_mariadb(conn);
+      long long int last_id = h_last_insert_id_mariadb(conn);
       if (last_id > 0) {
         data = h_new_data_int(last_id);
       } else {
@@ -344,7 +349,7 @@ struct _h_data * h_query_last_insert_id(const struct _h_connection * conn) {
 #endif
 #ifdef _HOEL_PGSQL
     } else if (conn->type == HOEL_DB_TYPE_PGSQL) {
-      int last_id = h_last_insert_id_pgsql(conn);
+      long long int last_id = h_last_insert_id_pgsql(conn);
       if (last_id > 0) {
         data = h_new_data_int(last_id);
       } else {
@@ -423,9 +428,9 @@ int h_clean_result(struct _h_result * result) {
           return H_ERROR_MEMORY;
         }
       }
-      o_free(result->data[row]);
+      h_free(result->data[row]);
     }
-    o_free(result->data);
+    h_free(result->data);
     return H_OK;
   } else {
     return H_ERROR_PARAMS;
@@ -440,12 +445,12 @@ int h_clean_result(struct _h_result * result) {
 int h_clean_data(struct _h_data * data) {
   if (data != NULL) {
     if (data->type == HOEL_COL_TYPE_TEXT) {
-      o_free(((struct _h_type_text *)data->t_data)->value);
+      h_free(((struct _h_type_text *)data->t_data)->value);
     } else if (data->type == HOEL_COL_TYPE_BLOB) {
-      o_free(((struct _h_type_blob *)data->t_data)->value);
+      h_free(((struct _h_type_blob *)data->t_data)->value);
     }
     if (data->t_data != NULL) {
-      o_free(data->t_data);
+      h_free(data->t_data);
     }
     return H_OK;
   } else {
@@ -461,7 +466,7 @@ int h_clean_data(struct _h_data * data) {
 int h_clean_data_full(struct _h_data * data) {
   if (data != NULL) {
     h_clean_data(data);
-    o_free(data);
+    h_free(data);
     return H_OK;
   } else {
     return H_ERROR_PARAMS;
@@ -479,7 +484,7 @@ struct _h_data * h_new_data_int(const long long int value) {
     data->t_data = o_malloc(sizeof(struct _h_type_int));
     if (data->t_data == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for data->t_data");
-      o_free(data);
+      h_free(data);
       return NULL;
     }
     data->type = HOEL_COL_TYPE_INT;
@@ -501,7 +506,7 @@ struct _h_data * h_new_data_double(const double value) {
     data->t_data = o_malloc(sizeof(struct _h_type_double));
     if (data->t_data == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for data->t_data");
-      o_free(data);
+      h_free(data);
       return NULL;
     }
     data->type = HOEL_COL_TYPE_DOUBLE;
@@ -523,14 +528,14 @@ struct _h_data * h_new_data_text(const char * value, const size_t length) {
     data->t_data = o_malloc(sizeof(struct _h_type_text));
     if (data->t_data == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for data->t_data");
-      o_free(data);
+      h_free(data);
       return NULL;
     }
     data->type = HOEL_COL_TYPE_TEXT;
     ((struct _h_type_text *)data->t_data)->value = o_malloc(length+1);
     if (((struct _h_type_text *)data->t_data)->value == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for data->t_data->value");
-      o_free(data);
+      h_free(data);
       return NULL;
     } else {
       memcpy(((struct _h_type_text *)data->t_data)->value, value, length);
@@ -556,7 +561,7 @@ struct _h_data * h_new_data_blob(const void * value, const size_t length) {
       data->t_data = o_malloc(sizeof(struct _h_type_blob));
       if (data->t_data == NULL) {
         y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for data");
-        o_free(data);
+        h_free(data);
         return NULL;
       }
       data->type = HOEL_COL_TYPE_BLOB;
@@ -564,7 +569,7 @@ struct _h_data * h_new_data_blob(const void * value, const size_t length) {
       ((struct _h_type_blob *)data->t_data)->value = o_malloc(length);
       if (((struct _h_type_blob *)data->t_data)->value == NULL) {
         y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for t_data->value");
-        o_free(data);
+        h_free(data);
         return NULL;
       } else {
         memcpy(((struct _h_type_blob *)data->t_data)->value, value, length);
@@ -609,7 +614,7 @@ struct _h_data * h_new_data_datetime(const struct tm * datetime) {
       
       if (data->t_data == NULL) {
         y_log_message(Y_LOG_LEVEL_ERROR, "Hoel - Error allocating memory for data->t_data");
-        o_free(data);
+        h_free(data);
         return NULL;
       }
       ((struct _h_type_datetime *)data->t_data)->value = * datetime;
@@ -627,8 +632,8 @@ struct _h_data * h_new_data_datetime(const struct tm * datetime) {
  */
 int h_clean_connection(struct _h_connection * conn) {
   if (conn != NULL) {
-    o_free(conn->connection);
-    o_free(conn);
+    h_free(conn->connection);
+    h_free(conn);
     return H_OK;
   } else {
     return H_ERROR_PARAMS;
